""" A simple command line interface wrapper around cookiecutter which uses the template defined in this project. """
from pathlib import Path
import sys

from cookiecutter.main import cookiecutter


TEMPLATES_DIRECTORY = Path(__file__).parent / "templates"


def run(template: str = "default") -> None:
    """ Generate a new project using given template """
    template_path = TEMPLATES_DIRECTORY / template
    if not template_path.is_dir():
        print(f"Template not found: {template}", file=sys.stderr)
        sys.exit(1)
    print(f"Creating new project using captain-cookie {template} template.\n")
    cookiecutter(str(template_path))



""" A dummy conftest to illustrate pytest configuration """

from pytest import fixture

from {{ cookiecutter.project_slug }} import __version__


@fixture
def version() -> str:
    """ Return the current version of the {{ cookiecutter.project_slug }} package """
    return __version__



""" Command Line Interface implementation for the {{ cookiecutter.project_slug }} package.

Two flags are available for the command line tool, I.E:

  - version: When version flag is passed (`--version`) the application will print the version and exit successfully.

  - verbose: When verbose flag is passed ( `--verbose`) the application state will be updated and verbose key will be set to True.
             Each command can check if verbose state is enabled and act accordingly.

The list of command can be printed with  {{ cookiecutter.cli_name }} --help.
"""

import typer

from .. import __version__


app = typer.Typer()


state = {"verbose": False}


def version_callback(value: bool) -> None:
    """ Print version and exit when given value is True. """
    if value:
        typer.echo(f"{{ cookiecutter.cli_name }} version: {__version__}")
        raise typer.Exit()


@app.callback()
def base(
    version: bool = typer.Option(
        False, "--version", callback=version_callback, is_eager=True
    ),
    verbose: bool = False,
) -> None:
    """ {{ cookiecutter.project_name }} command line interface. """
    if verbose:
        state["verbose"] = True


@app.command()
def hello(
    name: str = typer.Option(
        default="world", help="The name to greet. If not specified default to 'world'"
    )
):
    """ Say hello to given name. """
    if state["verbose"]:
        typer.echo("DEBUG: Running command 'hello'")
    typer.echo(f"Hello {name}")



""" Tasks for {{ cookiecutter.project_slug }} project. """

from pathlib import Path
from shutil import rmtree

from invoke import task


PROJECT_DIR = Path(__file__).parent
SRC_DIR = PROJECT_DIR / "src"
TESTS_DIR = PROJECT_DIR / "tests"

src_files = " ".join([str(path) for path in SRC_DIR.glob("**/*.py")])
test_files = " ".join([str(path) for path in TESTS_DIR.glob("**/*.py")])


@task
def doc(c):
    """ Serve the documentation. """
    c.run("mkdocs serve")


@task
def lint(c):
    """ Lint the code sources. """
    c.run(f"flake8 {src_files} {test_files} tasks.py")


@task
def format(c):
    """ Format the code sources. """

    c.run(f"black {src_files} {test_files} tasks.py")
    c.run(f"isort {src_files} {test_files} tasks.py")


@task
def build(c, doc=True):
    """ Build the user documentation. """
    if doc:
        c.run(f"mkdocs build -d {PROJECT_DIR / 'dist' / 'docs'}")
    c.run("poetry build")


@task
def clean(c):
    """ Clean the build artefacts. """
    try:
        rmtree(PROJECT_DIR / "dist")
    except FileNotFoundError:
        pass


@task
def test(c):
    """ Run the unit tests. """
    c.run("pytest")



#!/usr/bin/env python
""" Hooks ran after project generation:

- remove_cli_dir: Remove the src/<PROJECT_SLUG>/cli directory when create_cli is set to False
"""
from pathlib import Path
from shutil import rmtree
from subprocess import check_call
import sys
from distutils.util import strtobool


PROJECT_SLUG: str = "{{ cookiecutter.project_slug }}"
CREATE_CLI: int = strtobool("{{ cookiecutter.create_cli }}")


def remove_cli_dir(should_create: bool) -> None:
    """ Remove the CLI directory when remove argument is True. """
    if not should_create:
        cli_subpackage = Path("src") / PROJECT_SLUG / "cli"
        rmtree(cli_subpackage)


def init_git() -> None:
    """ Init a git repository. """
    try:
        check_call(["git", "init"])
    except Exception:
        print("Error encountered during 'git init'. Do you have git installed ?", file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":

    remove_cli_dir(CREATE_CLI)
    init_git()



#!/usr/bin/env python
""" Hooks ran before project generation:

- validate_project_slug: Ensure that project_slug is a valid python module name
"""
import re
import sys


MODULE_REGEX = r"^[_a-zA-Z][_a-zA-Z0-9]+$"
MODULE_NAME: str = "{{ cookiecutter.project_slug}}"


def validate_project_slug(name: str) -> None:
    """ Ensure that project slug is a valid python module name """
    if not re.match(MODULE_REGEX, name):
        print(
            f"ERROR: The project slug {name} is not a valid Python module name."
            "Please do not use a - and use _ instead",
            file=sys.stderr)
        #Exit to cancel project
        sys.exit(1)

if __name__ == "__main__":

    validate_project_slug(MODULE_NAME)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['fastrepo',
 'fastrepo.templates.default.hooks',
 'fastrepo.templates.default.{{cookiecutter.project_slug}}',
 'fastrepo.templates.default.{{cookiecutter.project_slug}}.src.{{cookiecutter.project_slug}}',
 'fastrepo.templates.default.{{cookiecutter.project_slug}}.src.{{cookiecutter.project_slug}}.cli',
 'fastrepo.templates.default.{{cookiecutter.project_slug}}.tests']

package_data = \
{'': ['*'],
 'fastrepo': ['templates/default/*',
              'templates/default/{{cookiecutter.project_slug}}/docs/*',
              'templates/default/{{cookiecutter.project_slug}}/docs/apidoc/*',
              'templates/default/{{cookiecutter.project_slug}}/docs/contributing/*',
              'templates/default/{{cookiecutter.project_slug}}/docs/css/*',
              'templates/default/{{cookiecutter.project_slug}}/docs/js/*',
              'templates/default/{{cookiecutter.project_slug}}/docs/tutorials/*']}

install_requires = \
['cookiecutter>=1.7.2,<2.0.0']

entry_points = \
{'console_scripts': ['create-repo = fastrepo:run']}

setup_kwargs = {
    'name': 'fastrepo',
    'version': '0.1.0',
    'description': 'Create new project repositories',
    'long_description': None,
    'author': 'gcharbon',
    'author_email': 'guillaume.charbonnier@capgemini.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



""" A dummy test to demonstrate pytest fixture usage """


def test_current_version(version):
    """ Receive current version from fixture and check value.
    
    In a real test you might want to do some operation on the fixture value
    before asserting a condition.
    """
    assert version == "0.1.0"
